import {type Session, rpc} from './api';
import {type Writable, writable} from 'svelte/store';
import {filesize} from 'filesize';

export * as api from './api';
export {rpc} from './api';

export interface UploadFile {
    id: number,
    name: string,
    full_name: string,
    file: File,
    overwrite: boolean
}

export const show_working = writable<boolean>(false);
export const info_banner = writable<string>('');
export const error_banner = writable<string>('');

export const token = writable<string|null>(localStorage.getItem('token'));
export const session: { s: Writable<Session|null>, update: (token: string|null) => Promise<void> } = {
    s: writable(null),
    update: async (t: string|null) => {
        if (t == null) {
            session.s.set(null);
            return;
        }
        const s = await rpc.sessionInfo();
        if (!s)
            token.set(null);
        else
            session.s.set(s);
    }
};
token.subscribe(t => rpc.token = t ?? '');
token.subscribe(t => session.update(t));

token.subscribe(v => {
    if (v == null)
        localStorage.removeItem('token');
    else
        localStorage.setItem('token', v);
})

export async function workingWrapper<T>(fn: () => Promise<T>): Promise<T|null> {
    let r = null;
    info_banner.set('');
    error_banner.set('');
    show_working.set(true);
    try {
        r = await fn();
    } catch (e) {
        error_banner.set(`Error while making request: ${e}`);
        token.set(null);
    }
    show_working.set(false);
    return r;
}

export async function workingWrapperO(fn: () => Promise<string|undefined>): Promise<boolean> {
    const resp = await workingWrapper(fn);
    if (resp)
        error_banner.set(resp);
    return resp == undefined;
}

export async function workingWrapperR<T>(fn: () => Promise<{
    e?: string,
    o?: T
} | undefined>): Promise<T|null> {
    const resp = await workingWrapper(fn);
    if (!resp)
        return null;
    if (resp.e != null)
        error_banner.set(resp.e);
    return resp.o as unknown as T;
}

export async function download<T extends {id:number, file:boolean}>(nodes: T[]) {
    const form = document.createElement('form');
    form.method = 'POST';
    form.target = '_blank';
    form.innerHTML = `<input type="hidden" name="token" value="${rpc.token}">`;
    if (nodes.length == 1 && nodes[0] && nodes[0].file) {
        form.action = '/api/public/download';
        form.innerHTML += `<input type="hidden" name="node" value="${nodes[0].id}">`;
    } else {
        form.action = '/api/public/download_multi';
        form.innerHTML += `<input type="hidden" name="nodes" value="${nodes.map(n => n.id).join('.')}">`;
        const resp = await workingWrapper(() => rpc.getNodesSize(nodes.map(v => v.id)));
        if (!resp)
            return;
        info_banner.set(`Estimated size: ${filesize(resp, {base: 2, standard: 'jedec'})}`);
    }
    document.body.appendChild(form);
    form.submit();
    document.body.removeChild(form);
}
