package de.mattv.fileserver.data;

import de.mattv.fileserver.Utils;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.nio.file.Files;
import java.util.HashSet;

@Slf4j
public class DataValidator {
    private static long errors = 0;

    public static void validateData() {
        log.info("Validating data");
        try {
            long lastUserId = -1;
            for (User user : Data.USERS.values()) {
                validateUser(user);
                lastUserId = Math.max(lastUserId, user.id);
            }
            if (lastUserId >= Data.nextUserId) {
                log.error("Next user id {} must be larger than the used ids {}", Data.nextUserId, lastUserId);
                errors++;
            }
        } catch (Throwable e) {
            Utils.crash(log, "Exception while validating data", e);
        }
        if (errors > 0)
            Utils.crash(log, "Encountered " + errors + " errors while validating data");
        log.info("Data validated");
    }

    private static void validateUser(User user) throws IOException {
        log.info("Validating user {}", user.id);

        long lastNodeId = 0;
        HashSet<Long> orphans = new HashSet<>();
        user.nodes.keys().asIterator().forEachRemaining(orphans::add);
        orphans.remove(0L);

        for (Node node : user.nodes.values()) {
            validateNode(node);
            lastNodeId = Math.max(lastNodeId, node.id);
            for (Node child : node.children)
                orphans.remove(child.id);
        }

        for (Long orphan : orphans) {
            log.error("Orphaned node {}", orphan);
            errors++;
        }
        if (lastNodeId >= user.nextNodeId.get()) {
            log.error("Next node id {} must be larger than the used ids {}", user.nextNodeId.get(), lastNodeId);
            errors++;
        }
    }

    private static void validateNode(Node node) throws IOException {
        if (node.isFile) {
            if (!node.children.isEmpty()) {
                log.error("Node {} is file and has children", node.id);
                errors++;
            }
            if (node.hasPreview && !node.previewFile.exists()) {
                log.warn("Missing preview file for node {}, regenerating", node.id);
                node.createThumbnail();
            }
            if (!node.file.exists()) {
                if (node.size == 0) {
                    log.warn("Created empty file for node {}", node.id);
                    if (!node.file.createNewFile()) {
                        log.error("Failed to create file {}", node.file.getAbsolutePath());
                        errors++;
                        return;
                    }
                } else {
                    log.error("Missing file for node {}", node.id);
                    errors++;
                    return;
                }
            }
            long realSize = Files.size(node.file.toPath());
            if (realSize != node.size) {
                log.error("Node {} size mismatch (Node, Disk) {} != {}", node.id, node.size, realSize);
                errors++;
            }
        }
    }
}
