package de.mattv.fileserver.routes;

import de.mattv.fileserver.data.Node;
import de.mattv.fileserver.data.Token;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.security.TokenService;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.PublicRestController;
import io.swagger.v3.oas.annotations.Operation;
import jakarta.servlet.http.HttpServletResponse;
import de.mattv.fileserver.util.NonNull;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.http.ContentDisposition;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PostMapping;

import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.util.ArrayDeque;
import java.util.Queue;
import java.util.zip.Deflater;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

@PublicRestController
public class DownloadMulti {
    private record Body(@NonNull String token, @NonNull long[] nodes) {}

    private static ZipEntry createEntry(String name, long size) {
        ZipEntry entry = new ZipEntry(name);
        entry.setSize(size);
        return entry;
    }

    @PostMapping(path = "/download_multi", consumes = {MediaType.APPLICATION_FORM_URLENCODED_VALUE})
    @Operation(hidden = true)
    private void downloadMulti(Body body, HttpServletResponse response) throws IOException {
        Token token = TokenService.getToken(body.token);
        if (token == null) {
            response.sendError(401, "Invalid token");
            return;
        }

        User user = token.getUser();
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            response.setContentType("application/zip");
            response.setHeader("Content-Disposition", ContentDisposition.attachment().filename("files.zip").build().toString());

            Queue<Pair<Node, String>> todo = new ArrayDeque<>();

            for (long id : body.nodes) {
                Node node = user.nodes.get(id);
                if (node != null)
                    todo.add(Pair.of(node, ""));
            }

            ZipOutputStream zip = new ZipOutputStream(response.getOutputStream());
            zip.setLevel(Deflater.NO_COMPRESSION);

            while (!todo.isEmpty()) {
                Pair<Node, String> entry = todo.remove();
                Node node = entry.getLeft();
                String path = entry.getRight() + node.name;

                if (node.isFile) {
                    zip.putNextEntry(createEntry(path, node.size));
                    try (InputStream is = Files.newInputStream(node.file.toPath())) {
                        is.transferTo(zip);
                    }
                    zip.closeEntry();
                } else {
                    path += "/";
                    zip.putNextEntry(createEntry(path, 0));
                    zip.closeEntry();
                    for (Node child : node.children)
                        todo.add(Pair.of(child, path));
                }
            }

            zip.finish();
            zip.flush();
            zip.close();
            response.flushBuffer();
        }
    }
}
