package de.mattv.fileserver.services;

import de.mattv.fileserver.Utils;
import de.mattv.fileserver.data.Data;
import de.mattv.fileserver.data.User;
import jakarta.annotation.Nullable;
import de.mattv.fileserver.util.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.Instant;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

@Slf4j
@Service
public class RecoveryMailer {
    private static final ConcurrentHashMap<String, Pair<Long, Instant>> RECOVERY_KEYS = new ConcurrentHashMap<>();

    private static final Duration LIFETIME = Duration.ofMinutes(5);
    private static final int CODE_LENGTH = 16;

    private final Mailer mailer;

    public RecoveryMailer(@NonNull Mailer mailer) {
        this.mailer = mailer;
    }

    public void sendMail(@NonNull User user) {
        log.info("Sending recovery mail to: {}", user.name);
        String code = RandomStringUtils.random(CODE_LENGTH, false, true);
        RECOVERY_KEYS.put(code, Pair.of(user.id, Instant.now().plus(LIFETIME)));
        mailer.send(user.name, "Password recovery", "Your recovery key is: " + code + "\nIt is valid for " + LIFETIME.toMinutes() + " minutes.");
    }

    public @Nullable User checkCode(@NonNull String code) {
        if (code.length() != CODE_LENGTH)
            return null;
        Pair<Long, Instant> entry = RECOVERY_KEYS.remove(code);
        if (entry == null || Utils.instantExpired(entry.getRight()))
            return null;
        return Data.USERS.get(entry.getLeft());
    }

    @Scheduled(fixedRate = 2, timeUnit = TimeUnit.HOURS)
    private static void cleanExpiredKeys() {
        RECOVERY_KEYS.entrySet().removeIf(entry -> Utils.instantExpired(entry.getValue().getRight()));
    }
}
